﻿using ProjectSeaplane.DrawningObjects;
using ProjectSeaplane.Generics;
using ProjectSeaplane.MovementStrategy;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.Extensions.Logging;
using System.Xml.Linq;
using ProjectSeaplane.Exceptions;

namespace ProjectSeaplane
{
    public partial class FormPlaneCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly PlanesGenericStorage _storage;
        private readonly ILogger _logger;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormPlaneCollection(ILogger<FormPlaneCollection> logger)
        {
            InitializeComponent();
            _storage = new PlanesGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
            _logger = logger;

        }
        private void ButtonSortByType_Click(object sender, EventArgs e) => ComparePlanes(new PlaneCompareByType());

        private void ButtonSortByColor_Click(object sender, EventArgs e) => ComparePlanes(new PlaneCompareByColor());

        private void ComparePlanes(IComparer<DrawningPlane?> comparer)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            obj.Sort(comparer);
            pictureBoxCollection.Image = obj.ShowPlanes();
        }

        /// <summary>
        /// Заполнение listBoxObjects
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;
            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i].Name);
            }

            if (listBoxStorages.Items.Count > 0 && (index == -1 ||
                index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }
            else if (listBoxStorages.Items.Count > 0 && index > -1 &&
                index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }

        }
        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("Пустое название набора");
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            _logger.LogInformation($"Добавлен набор: {textBoxStorageName.Text}");
        }
        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxObjects_SelectedIndexChanged(object sender,
        EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowPlanes();
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                _logger.LogWarning("Удаление невыбранного набора");
                return;
            }
            string name = listBoxStorages.SelectedItem.ToString() ?? string.Empty;
            if (MessageBox.Show($"Удалить объект {name}?",
                "Удаление", MessageBoxButtons.YesNo,
                MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString()
                ?? string.Empty);
                ReloadObjects();
                _logger.LogInformation($"Удален набор: {name}");
            }
        }


        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddPlane_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            FormPlaneConfig form = new FormPlaneConfig();

            Action<DrawningPlane>? planeDelegate = new((m) =>
            {
                var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
                if (obj == null)
                {
                    _logger.LogWarning("Добавление пустого объекта");
                    return;
                }
                try
                {
                    _ = obj + m;

                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowPlanes();
                    _logger.LogInformation($"Добавлен объект в набор {listBoxStorages.SelectedItem.ToString()}");
                }
                catch (StorageOverflowException ex)
                {
                    MessageBox.Show(ex.Message);
                    _logger.LogWarning($"Не удалось добавить объект: {ex.Message}");
                }
                catch (ArgumentException ex)
                {
                    _logger.LogWarning($"Добавляемый объект уже существует в коллекции {listBoxStorages.SelectedItem.ToString() ?? string.Empty}");
                    MessageBox.Show("Добавляемый объект уже сущесвует в коллекции");

                }

            });
            form.AddEvent(planeDelegate);
            form.Show();
        }
        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemovePlane_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                _logger.LogWarning("Удаление объекта из несуществующего набора");
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo,
            MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            try
            {
                int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
                if (obj - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBoxCollection.Image = obj.ShowPlanes();
                    _logger.LogInformation($"Удален объект из набора {listBoxStorages.SelectedItem.ToString()}");

                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                    _logger.LogWarning($"Не удалось удалить объект из набора {listBoxStorages.SelectedItem.ToString()}");
                }
            }
            catch (PlaneNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogWarning($"{ex.Message} из набора {listBoxStorages.SelectedItem.ToString()}");

            }
            catch (Exception ex)
            {
                MessageBox.Show("Некорректные данные");
                _logger.LogWarning("Некорректные данные");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRefreshCollection_Click(object sender, EventArgs e)
        {
            {
                if (listBoxStorages.SelectedIndex == -1)
                {
                    return;
                }
                var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
                if (obj == null)
                {
                    return;
                }
                pictureBoxCollection.Image = obj.ShowPlanes();
            }
        }

        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Сохранение наборов в файл {saveFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не удалось сохранить наборы с ошибкой: {ex.Message}");
                }

            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog.FileName);
                    ReloadObjects();
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Загрузились наборы из файла {openFileDialog.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не загрузилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не удалось сохранить наборы с ошибкой: {ex.Message}");
                }

            }
        }

    }
}
